import datetime
import datetime as dt
from datetime import timezone
from datetime import timezone as tz

print(datetime.timezone(-1))
print(timezone.utc)
print(tz.utc)

print(datetime.timezone.utc)
print(dt.timezone.utc)
